@@ -106,8 +106,6 @@ class Displayable(renpy.object.Object):
                 continue
 
             i.find_focusable(callback, focus_name)
-            
-
 
     def focus(self, default=False):
         """
@@ -128,8 +126,6 @@ class Displayable(renpy.object.Object):
         if not self.activated:
             self.set_style_prefix(self.role + "idle_")
 
-
-
     def is_focused(self):
         return renpy.game.context().scene_lists.focused is self
 
@@ -646,6 +642,9 @@ class Display(object):
 
     @ivar next_frame: The time when the next frame should be drawn. In
     ms returned from pygame.time.get_ticks().
+
+    @ivar window_caption: The current window caption.
+
     """
 
 
@@ -715,20 +714,29 @@ class Display(object):
                 renpy.display.scale.image_load_unscaled(
                     renpy.loader.load(renpy.config.window_icon),
                     renpy.config.window_icon))
+
+        # If a window exists of the right size and flags, use it. Otherwise,
+        # make our own window.
+        old_window = pygame.display.get_surface()
+        if ((old_window is not None) and 
+            (old_window.get_size() == (width, height)) and
+            (old_window.get_flags() & FULLSCREEN == fsflag)):
             
-        # The window we display things in.
-        self.window = pygame.display.set_mode((width, height), fsflag, 32)
+            self.window = old_window
+                    
+        else:
+            self.window = pygame.display.set_mode((width, height), fsflag, 32)
 
         # Window title.
-        pygame.display.set_caption(renpy.config.window_title.encode("utf-8"))
-
-
+        self.window_caption = None
+        self.set_window_caption()
+        
         # Sample surface that all surfaces are created based on.
         sample = pygame.Surface((10, 10))
         self.sample_surface = sample.convert_alpha()
 
         pygame.event.set_grab(False)
-
+        
         # Load the mouse image, if any.
         if renpy.config.mouse:
             self.mouse = True
@@ -758,6 +766,14 @@ class Display(object):
         # Setup periodic event.
         pygame.time.set_timer(PERIODIC, PERIODIC_INTERVAL)
 
+    def set_window_caption(self):
+        caption = renpy.config.window_title + renpy.store._window_subtitle
+        if caption == self.window_caption:
+            return
+
+        self.window_caption = caption
+        pygame.display.set_caption(caption.encode("utf-8"))
+
     def can_redraw(self, first_pass):
         """
         Uses the framerate to determine if we can and should redraw.
@@ -850,7 +866,7 @@ class Display(object):
             visible = False
         else:
             visible = renpy.store.mouse_visible
-
+            
         # Deal with a hardware mouse, the easy way.
         if not self.mouse:
 
@@ -879,6 +895,9 @@ class Display(object):
 
         pos = pygame.mouse.get_pos()
 
+        if not renpy.game.interface.focused:
+            pos = None
+            
         if (pos == self.mouse_location and
             show_mouse and
             info == self.mouse_info):
@@ -889,8 +908,8 @@ class Display(object):
 
         if self.mouse_location:
             updates.append(self.hide_mouse())
-
-        if visible and pos:
+            
+        if visible and pos and renpy.game.interface.focused:
             updates.append(self.show_mouse(pos, info))
             
         return updates
@@ -1067,6 +1086,10 @@ class Interface(object):
 
         self.time_event = pygame.event.Event(TIMEEVENT)
 
+        # Are we focused?
+        self.focused = True
+
+        
         # Properties for each layer.
         self.layer_properties = { }
 
@@ -1211,13 +1234,13 @@ class Interface(object):
             self.last_event = rv
             return rv
 
-        while renpy.display.im.cache.needs_preload():
-            ev = pygame.event.poll()
-            if ev.type != NOEVENT:
-                self.last_event = ev
-                return ev
+#         while renpy.display.im.cache.needs_preload():
+#             ev = pygame.event.poll()
+#             if ev.type != NOEVENT:
+#                 self.last_event = ev
+#                 return ev
 
-            renpy.display.im.cache.preload()
+#             renpy.display.im.cache.preload()
             
         try:
             cpu_idle.set()            
@@ -1259,6 +1282,9 @@ class Interface(object):
 
         # These things can be done once per interaction.
 
+        import time
+        start = time.time()
+        
         preloads = self.preloads
         self.preloads = [ ]
 
@@ -1287,7 +1313,7 @@ class Interface(object):
             self.transition_from = { }
                 
             self.restart_interaction = True
-        
+            
     def interact_core(self,
                       show_mouse=True,
                       trans_pause=False,
@@ -1317,7 +1343,7 @@ class Interface(object):
 
         # The global one.
         self.suppress_transition = False
-
+        
         # Figure out transitions.
         for k in self.transition:
             if k not in self.old_scene:
@@ -1326,9 +1352,14 @@ class Interface(object):
             self.ongoing_transition[k] = self.transition[k]            
             self.transition_from[k] = self.old_scene[k]
             self.transition_time[k] = None
-
+                    
         self.transition.clear()
-                
+
+        if suppress_transition:
+            self.ongoing_transition.clear()
+            self.transition_from.clear()
+            self.transition_time.clear()
+        
         ## Safety condition, prevents deadlocks.
         if trans_pause:
             if not self.ongoing_transition:
@@ -1355,6 +1386,9 @@ class Interface(object):
         for i in renpy.config.interact_callbacks:
             i()
 
+        # Set the window caption.
+        self.display.set_window_caption()
+            
         # Tick time forward.
         renpy.display.im.cache.tick()
 
@@ -1363,7 +1397,6 @@ class Interface(object):
         
         # Clear some events.
         pygame.event.clear((MOUSEMOTION, PERIODIC,
-                            MOUSEBUTTONUP, MOUSEBUTTONDOWN,
                             TIMEEVENT, REDRAW))
 
         # Add a single TIMEEVENT to the queue.
@@ -1533,6 +1566,9 @@ class Interface(object):
                     # If we have a movie, start showing it.
                     suppress_blit = renpy.display.video.interact()
 
+                    # Clean out the redraws, if we have to.
+                    renpy.display.render.kill_redraws()
+                    
                     # Draw the screen.
                     self.frame_time = get_time()
 
@@ -1568,12 +1604,12 @@ class Interface(object):
                     pygame.time.set_timer(REDRAW, 0)
                     pygame.event.clear([REDRAW])
                     old_redraw_time = None
-
-
+                    
                 # Draw the mouse, if it needs drawing.
                 if show_mouse:
                     self.display.update_mouse()
                     
+
                 # See if we want to restart the interaction entirely.
                 if self.restart_interaction:                    
                     return True, None
@@ -1581,6 +1617,18 @@ class Interface(object):
                 # Determine if we need a redraw.
                 needs_redraw = needs_redraw or renpy.display.render.process_redraws()
 
+                # Predict images, if we haven't done so already.
+                if not did_prediction and not self.event_peek():                    
+                    if not renpy.config.load_before_transition:
+                        root_widget.predict(renpy.display.im.cache.preload_image)
+
+                    for w in preloads:
+                        w.predict(renpy.display.im.cache.preload_image)
+
+                    renpy.game.context().predict(renpy.display.im.cache.preload_image)
+                    renpy.display.im.cache.preload()
+                    did_prediction = True
+
                 # If we need to redraw again, do it if we don't have an
                 # event going on.
                 if needs_redraw and not self.event_peek():
@@ -1588,17 +1636,6 @@ class Interface(object):
                         self.profile_time = get_time()
                     continue
 
-                # Predict images, if we haven't done so already.
-                if not did_prediction and not self.event_peek():
-                    root_widget.predict(renpy.display.im.cache.preload_image)
-
-                    for w in preloads:
-                        w.predict(renpy.display.im.cache.preload_image)
-
-                    renpy.game.context().predict(renpy.display.im.cache.preload_image)
-                    did_prediction = True
-
-
                 try:
 
                     # Times until events occur.
@@ -1671,7 +1708,7 @@ class Interface(object):
 
                         if ev2 and ev2.type not in (NOEVENT, PERIODIC, REDRAW, QUIT):
                             ev = self.event_poll()
-
+                            
                     # Handle redraw timeouts.
                     if ev.type == REDRAW:
                         old_redraw_time = None
@@ -1701,7 +1738,7 @@ class Interface(object):
 
                     # Handle skipping.
                     renpy.display.behavior.skipping(ev)
-
+                    
                     # Handle quit specially for now.
                     if ev.type == QUIT:
                         if renpy.game.script.has_label("_confirm_quit") and not self.quick_quit:
@@ -1714,19 +1751,23 @@ class Interface(object):
                     # Merge mousemotion events.
                     if ev.type == MOUSEMOTION:
                         evs = pygame.event.get([MOUSEMOTION])
-
                         if len(evs):
                             ev = evs[-1]
 
-
+                        self.focused = True
+                            
+                    # Handle focus notifications.
+                    if ev.type == ACTIVEEVENT:
+                        if ev.state & 1:
+                            self.focused = ev.gain
+                            
                     # x, y = getattr(ev, 'pos', (0, 0))
                     x, y = pygame.mouse.get_pos()
                     x -= self.display.screen_xoffset
                     y -= self.display.screen_yoffset
-
+                    
                     self.event_time = end_time = get_time()
 
-
                     # Handle the event normally.
                     rv = renpy.display.focus.mouse_handler(ev, x, y)
 
@@ -1738,7 +1779,7 @@ class Interface(object):
 
                     if rv is not None:
                         break
-
+                    
                     # Handle displayable inspector.
                     if renpy.config.inspector and renpy.display.behavior.inspector(ev):
                         l = self.display.surftree.main_displayables_at_point(x, y, renpy.config.transient_layers + renpy.config.overlay_layers)
@@ -1774,6 +1815,9 @@ class Interface(object):
             for i in renpy.config.overlay_layers:
                 scene_lists.clear(i)
 
+            # Stop ongoing preloading.
+            renpy.display.im.cache.end_tick()
+                
             # We no longer disable periodic between interactions.
             # pygame.time.set_timer(PERIODIC, 0)
 
