@@ -106,7 +106,7 @@ class Container(renpy.display.core.Displayable):
         """
 
         if child is None:
-            return
+            raise Exception("Trying to add None child to container.")
 
         self.children.append(child)
         self.offsets.append((0, 0))
@@ -127,6 +127,7 @@ class Container(renpy.display.core.Displayable):
         offsets = self.offsets
         
         for i in xrange(len(self.children)-1, -1, -1):
+
             d = children[i]
             xo, yo = offsets[i]
 
@@ -223,6 +224,7 @@ class Position(Container):
         """
 
         super(Position, self).__init__(style=style, **properties)
+        child = renpy.easy.displayable(child)
         self.add(child)
 
     def render(self, width, height, st, at):
@@ -382,7 +384,7 @@ class MultiBox(Container):
         super(MultiBox, self).add(widget)
         self.start_times.append(start_time)
         self.anim_times.append(anim_time)
-
+        
     def append_scene_list(self, l):
         for tag, zo, start, anim, d in l:
             self.add(d, start, anim)
@@ -423,7 +425,7 @@ class MultiBox(Container):
             
             rv = renpy.display.render.Render(width, height, layer_name=self.layer_name)
 
-        
+
             for child, start, anim in zip(self.children, self.start_times, self.anim_times):
 
                 cst = t - start
@@ -637,7 +639,9 @@ class Window(Container):
     def __init__(self, child, style='window', **properties):
 
         super(Window, self).__init__(style=style, **properties)
-        self.add(child)
+        child = renpy.easy.displayable_or_none(child)
+        if child is not None:
+            self.add(child)
 
     def visit(self):
         return [ self.style.background ] + self.children
@@ -796,7 +800,7 @@ class Motion(Container):
         seconds.
         """
 
-        child = renpy.easy.displayable(child)
+        child = renpy.easy.displayable_or_none(child)
 
         if child is None:
             child = new_widget
@@ -806,8 +810,9 @@ class Motion(Container):
 
         super(Motion, self).__init__(style=style, **properties)
 
-        self.child = child 
-        self.children = [ child ]
+        if child is not None:
+            self.add(child)
+
         self.function = function
         self.period = period
         self.repeat = repeat
@@ -834,7 +839,6 @@ class Motion(Container):
         else:
             t = st
 
-
         if self.delay and t >= self.delay:
             t = self.delay            
             if self.repeat:
@@ -910,7 +914,7 @@ class Interpolate(object):
             if c is not None and isinstance(b, float):
                 b = int(b * c)
             
-            rv = (1.0 - t) * a + t * b
+            rv = a + t * (b - a)
             
             if isinstance(a, int) and isinstance(b, int):
                 return int(rv)
@@ -1106,6 +1110,8 @@ class Zoom(renpy.display.core.Displayable):
     def __init__(self, size, start, end, time, child,
                  after_child=None, time_warp=None,
                  bilinear=True, opaque=True,
+                 anim_timebase=False,
+                 repeat=False,
                  **properties):
         """
         @param size: The size that the rectangle is scaled to, a
@@ -1141,8 +1147,9 @@ class Zoom(renpy.display.core.Displayable):
         self.start = start
         self.end = end
         self.time = time
-        self.child = child            
         self.done = 0.0
+        self.child = child
+        self.repeat = repeat
         
         if after_child:
             self.after_child = renpy.easy.displayable(after_child)
@@ -1155,18 +1162,27 @@ class Zoom(renpy.display.core.Displayable):
         self.time_warp = time_warp
         self.bilinear = bilinear and renpy.display.module.can_bilinear_scale
         self.opaque = opaque
-
+        self.anim_timebase = anim_timebase
+        
 
     def visit(self):
         return [ self.child, self.after_child ]
 
     def render(self, width, height, st, at):
 
+        if self.anim_timebase:
+            t = at
+        else:
+            t = st
+        
         if self.time:
-            done = min(st / self.time, 1.0)
+            done = min(t / self.time, 1.0)
         else:
             done = 1.0
 
+        if self.repeat:
+            done = done % 1.0
+            
         self.done = done
 
         if self.after_child and done == 1.0:
@@ -1225,7 +1241,7 @@ def zoom_core(rend, surf, rect, neww, newh, bilinear, opaque):
             renpy.display.module.bilinear_scale(surf, dest,
                                                 sx, sy, sw, sh,
                                                 -x, -y, neww, newh)
-
+            
         rv = renpy.display.render.Render(neww, newh, draw_func=draw, opaque=True)
         
 
@@ -1234,13 +1250,17 @@ def zoom_core(rend, surf, rect, neww, newh, bilinear, opaque):
         if bilinear:
             sx, sy, sw, sh = rect
 
-            scalesurf = pygame.Surface((neww, newh), 0, surf)
+            scalesurf = pygame.Surface((neww, newh), surf.get_flags(), surf)
+
             renpy.display.module.bilinear_scale(surf, scalesurf,
                                                 sx, sy, sw, sh,
                                                 0, 0, neww, newh)
         else:
-            scalesurf = pygame.transform.scale(surf, (neww, newh))
 
+            renpy.display.render.blit_lock.acquire()
+            scalesurf = pygame.transform.scale(surf, (neww, newh))
+            renpy.display.render.blit_lock.release()
+            
         renpy.display.render.mutated_surface(scalesurf)
 
         rv = renpy.display.render.Render(neww, newh)
@@ -1255,27 +1275,9 @@ class FactorZoom(renpy.display.core.Displayable):
     def __init__(self, start, end, time, child,
                  after_child=None, time_warp=None,
                  bilinear=True, opaque=True,
+                 anim_timebase=False,
+                 repeat=False,
                  **properties):
-        """
-        @param start: The start scaling factor.
-
-        @param end: The end scaling factor.
-
-        @param time: The amount of time it will take to
-        from the start to the end scaling factors.
-
-        @param child: The child displayable.
-
-        @param after_child: If present, a second child
-        widget. This displayable will be rendered after the zoom
-        completes. Use this to snap to a sharp displayable after
-        the zoom is done.
-
-        @param time_warp: If not None, this is a function that takes a
-        fraction of the period (between 0.0 and 1.0), and returns a
-        new fraction of the period. Use this to warp time, applying
-        acceleration and deceleration to motions.
-        """
 
         super(FactorZoom, self).__init__(**properties)
 
@@ -1284,7 +1286,8 @@ class FactorZoom(renpy.display.core.Displayable):
         self.start = start
         self.end = end
         self.time = time
-        self.child = child                    
+        self.child = child 
+        self.repeat = repeat
         
         if after_child:
             self.after_child = renpy.easy.displayable(after_child)
@@ -1298,17 +1301,26 @@ class FactorZoom(renpy.display.core.Displayable):
         self.bilinear = bilinear and renpy.display.module.can_bilinear_scale
         self.opaque = opaque
         self.done = 0.0
+        self.anim_timebase = anim_timebase
         
     def visit(self):
         return [ self.child, self.after_child ]
 
     def render(self, width, height, st, at):
 
+        if self.anim_timebase:
+            t = at
+        else:
+            t = st
+        
         if self.time:
-            done = min(st / self.time, 1.0)
+            done = min(t / self.time, 1.0)
         else:
             done = 1.0
 
+        if self.repeat:
+            done = done % 1.0
+            
         self.done = done
             
         if self.after_child and done == 1.0:
@@ -1337,11 +1349,95 @@ class FactorZoom(renpy.display.core.Displayable):
         return rv
 
     def event(self, ev, x, y, st):
-        if self.done == 1.0:
-            return self.child.event(ev, x, y, st)
+        if self.done == 1.0 and self.after_child:
+            return self.after_child.event(ev, x, y, st)
         else:
             return None
 
+class SizeZoom(renpy.display.core.Displayable):
+
+    def __init__(self, start, end, time, child,
+                 after_child=None, time_warp=None,
+                 bilinear=True, opaque=True,
+                 anim_timebase=False,
+                 repeat=False,
+                 **properties):
+
+        super(SizeZoom, self).__init__(**properties)
+
+        child = renpy.easy.displayable(child)
+
+        self.start = start
+        self.end = end
+        self.time = time        
+        self.child = child 
+        self.repeat = repeat
+        
+        if after_child:
+            self.after_child = renpy.easy.displayable(after_child)
+        else:
+            if self.end == (1.0, 1.0):
+                self.after_child = child
+            else:
+                self.after_child = None
+        
+        self.time_warp = time_warp
+        self.bilinear = bilinear and renpy.display.module.can_bilinear_scale
+        self.opaque = opaque
+        self.done = 0.0
+        self.anim_timebase = anim_timebase
+        
+    def visit(self):
+        return [ self.child, self.after_child ]
+
+    def render(self, width, height, st, at):
+
+        if self.anim_timebase:
+            t = at
+        else:
+            t = st
+        
+        if self.time:
+            done = min(t / self.time, 1.0)
+        else:
+            done = 1.0
+
+        if self.repeat:
+            done = done % 1.0
+            
+        self.done = done
+            
+        if self.after_child and done == 1.0:
+            self.child = self.after_child
+            return renpy.display.render.render(self.after_child, width, height, st, at)
+
+        if self.time_warp:
+            done = self.time_warp(done)
+
+        rend = renpy.display.render.render(self.child, width, height, st, at)
+        surf = rend.pygame_surface()
+
+        sx, sy = self.start
+        ex, ey = self.end
+
+        neww = int(sx + (ex - sx) * done)
+        newh = int(sy + (ey - sy) * done)
+        oldw, oldh = surf.get_size()
+
+        rv = zoom_core(rend, surf, (0, 0, oldw, oldh), neww, newh, self.bilinear, self.opaque)
+        
+        if done < 1.0:
+            renpy.display.render.redraw(self, 0)
+
+        self.done = done
+            
+        return rv
+
+    def event(self, ev, x, y, st):
+        if self.done == 1.0 and self.after_child:
+            return self.after_child.event(ev, x, y, st)
+        else:
+            return None
 
         
 def dynamic_displayable_compat(st, at, expr):
@@ -1558,8 +1654,15 @@ class RotoZoom(renpy.display.core.Displayable):
         else:
             zoom_time = st
 
-        rot_time /= self.rot_delay
-        zoom_time /= self.zoom_delay
+        if self.rot_delay == 0:
+            rot_time = 1.0
+        else:
+            rot_time /= self.rot_delay
+
+        if self.zoom_delay == 0:
+            zoom_time = 1.0
+        else:
+            zoom_time /= self.zoom_delay
 
         if self.rot_repeat:
             rot_time %= 1.0
@@ -1664,8 +1767,9 @@ class Viewport(Container):
                  **properties):
 
         super(Viewport, self).__init__(style=style, **properties)
-        child = renpy.easy.displayable(child)
-        self.add(child)
+        child = renpy.easy.displayable_or_none(child)
+        if child is not None:
+            self.add(child)
 
         if xadjustment is None:
             self.xadjustment = renpy.display.behavior.Adjustment(1, 0)
