@@ -34,8 +34,14 @@ except:
 import time
 import renpy
 import pygame
+import threading
 from pygame.constants import *
 
+# We grab the blit lock each time it is necessary to blit
+# something. This allows call to the pygame.transform functions to
+# disable blitting, should it prove necessary.
+blit_lock = threading.Condition()
+
 # We only cache a single solid... but that should be enough to handle
 # some important cases, like button and window backgrounds.
 class SolidCache(object):
@@ -148,6 +154,10 @@ def render(d, width, height, st, at):
 # A list of (when, widget) for redraws.
 redraw_queue = [ ]
 
+# The set of dead displayables.
+dead_displayables = set()
+
+
 def process_redraws():
     """
     Processes pending redraws. Returns True if a redraw is needed.
@@ -157,7 +167,6 @@ def process_redraws():
     redraw_queue.sort()
 
     i = 0
-    dead_widgets = set()
     now = renpy.display.core.get_time()
 
     for when, widget in redraw_queue:
@@ -167,18 +176,23 @@ def process_redraws():
 
         i += 1
 
-        dead_widgets.add(widget)
+        dead_displayables.add(widget)
 
-    if not dead_widgets:
+    if i:
+        redraw_queue = redraw_queue[i:]
+        
+    if dead_displayables:
+        return True
+    else:
         return False
-
-    redraw_queue = redraw_queue[i:]
+    
+def kill_redraws():
 
     for (widget, width, height), render in old_renders.items():
-        if widget in dead_widgets:
+        if widget in dead_displayables:
             render.kill()
-    
-    return True
+
+    dead_displayables.clear()
 
 def redraw_time():
     """
@@ -749,7 +763,10 @@ class Render(object):
                 if winblit:
                     what = cacheget(id(what), what)
 
+                blit_lock.acquire()
                 dest.blit(what, (x, y))
+                blit_lock.release()
+
                 continue
 
             
@@ -981,8 +998,8 @@ class Render(object):
             return False
         
         for xo, yo, source in self.blittables:
-            xx = x - xo
-            yy = y - yo
+            xx = int(x - xo)
+            yy = int(y - yo)
 
             if xx < 0 or yy < 0:
                 continue
@@ -1001,7 +1018,7 @@ class Render(object):
         return False
                 
     def main_displayables_at_point(self, x, y, layers, depth=None):
-        
+
         if x < 0 or y < 0 or x >= self.width or y >= self.height:
             return [ ]
 
@@ -1018,9 +1035,17 @@ class Render(object):
 
         for xo, yo, child in self.main_child_renders:
             rv.extend(child.main_displayables_at_point(x - xo, y - yo, layers, depth))
-
-        return rv
             
+        return rv
+
+    def canvas(self):
+        surf = pygame.Surface((self.width, self.height), 0, renpy.game.interface.display.sample_surface)
+        mutated_surface(surf)
+
+        self.blit(surf, (0, 0))
+
+        return Canvas(surf)
+        
         
 # Determine if a surface is fullscreen or not.
 def is_fullscreen(surf, x, y, wh):
@@ -1060,3 +1085,89 @@ def is_fullscreen(surf, x, y, wh):
     surf.fullscreen[xywh] = False
     return False
 
+
+class Canvas(object):
+
+    def __init__(self, surf):
+        self.surf = surf
+        
+    def rect(self, color, rect, width=0):
+        blit_lock.acquire()
+        pygame.draw.rect(self.surf,
+                            renpy.easy.color(color),
+                            rect,
+                            width)
+        blit_lock.release()
+
+    def polygon(self, color, pointlist, width=0):
+        blit_lock.acquire()
+        pygame.draw.polygon(self.surf,
+                            renpy.easy.color(color),
+                            pointlist,
+                            width)
+        blit_lock.release()
+
+    def circle(self, color, pos, radius, width=0):
+        blit_lock.acquire()
+        pygame.draw.circle(self.surf,
+                           renpy.easy.color(color),
+                           pos,
+                           radius,
+                           width)
+        blit_lock.release()
+
+    def ellipse(self, color, rect, width=0):
+        blit_lock.acquire()
+        pygame.draw.ellipse(self.surf,
+                           renpy.easy.color(color),
+                           rect,
+                           width)
+        blit_lock.release()
+
+
+    def arc(self, color, rect, start_angle, stop_angle, width=1):
+        blit_lock.acquire()
+        pygame.draw.arc(self.surf,
+                        renpy.easy.color(color),
+                        rect,
+                        start_angle,
+                        stop_angle,
+                        width)
+        blit_lock.release()
+
+
+    def line(self, color, start_pos, end_pos, width=1):
+        blit_lock.acquire()
+        pygame.draw.line(self.surf,
+                         renpy.easy.color(color),
+                         start_pos,
+                         end_pos,
+                         width)
+        blit_lock.release()
+
+    def lines(self, color, closed, pointlist, width=1):
+        blit_lock.acquire()
+        pygame.draw.lines(self.surf,
+                         renpy.easy.color(color),
+                         closed,
+                         pointlist,
+                         width)
+        blit_lock.release()
+    
+    def aaline(self, color, startpos, endpos, blend=1):
+        blit_lock.acquire()
+        pygame.draw.aaline(self.surf,
+                         renpy.easy.color(color),
+                         startpos,
+                         endpos,
+                         blend)
+        blit_lock.release()
+
+    def aalines(self, color, closed, pointlist, blend=1):
+        blit_lock.acquire()
+        pygame.draw.aalines(self.surf,
+                         renpy.easy.color(color),
+                         closed,
+                         pointlist,
+                         blend)
+        blit_lock.release()
