@@ -135,6 +135,22 @@ def image(name, img):
     img = renpy.easy.displayable(img)
 
     images[name] = img
+
+def copy_images(old, new):
+    if not isinstance(old, tuple):
+        old = tuple(old.split())
+
+    if not isinstance(new, tuple):
+        new = tuple(new.split())
+
+    lenold = len(old)
+        
+    for k in list(images.keys()):
+        if len(k) < lenold:
+            continue
+        
+        if k[:lenold] == old:
+            images[new + k[lenold:]] = images[k]
     
 def showing(name, layer='master'):
     """
@@ -692,12 +708,15 @@ def take_screenshot(scale=None):
     
     renpy.game.interface.take_screenshot(scale)
 
-def full_restart(reason="end_game"):
+def full_restart(transition=False, label="_invoke_main_menu"):
     """
     This causes a full restart of Ren'Py. 
     """
 
-    raise renpy.game.FullRestartException(reason)
+    if transition is False:
+        transition = renpy.config.end_game_transition
+    
+    raise renpy.game.FullRestartException((transition, label))
 
 def utter_restart():
     """
@@ -886,14 +905,13 @@ def get_filename_line():
     return n.filename, n.linenumber
 
 def shell_escape(s):
-    import sys
-    import re
-
-    if hasattr(sys, "winver"):
-        return s.replace("\\", "\\\\").replace("\"", "\\\"")
-    else:
-        return s.replace("\\", "\\\\").replace("'", "\\'")
-
+    s = s.replace("\\", "\\\\")
+    # s = s.replace("\'", "\\\'")
+    s = s.replace("\"", "\\\"")
+    s = s.replace("\$", "\\\$")
+    s = s.replace("\%", "\\\%")
+    s = s.replace("\`", "\\\`")
+    return s
 
 def launch_editor(filenames, line=1, transient=0):
     """
@@ -911,26 +929,30 @@ def launch_editor(filenames, line=1, transient=0):
     if not len(filenames):
         return
 
-    if hasattr(sys, "winver"):
-        shell = False
-        join = '" "'
-    else:
-        shell = True
-        join = "' '"
-
+    # join = '" "'
+    shell = True
+    
+#     if hasattr(sys, "winver"):
+#         shell = False
+#         # join = '" "'
+#     else:
+#         shell = True
+#         # join = "' '"
 
     filenames = [ shell_escape(os.path.normpath(i)) for i in filenames ]
     filename = filenames[0]
 
-    allfiles = join.join(filenames)
-    otherfiles = join.join(filenames[1:])
-                                
+    allfiles =  '" "'.join(filenames)
+    otherfiles = '" "'.join(filenames[1:])
+    
     subs = dict(filename=filename, line=line, allfiles=allfiles, otherfiles=otherfiles)
     if transient and (renpy.config.editor_transient is not None):
         cmd = renpy.config.editor_transient % subs
     else:
         cmd = renpy.config.editor % subs
 
+    cmd = cmd.replace('""', '')
+
     try:
         return subprocess.Popen(cmd, shell=shell)
     except:
@@ -1070,7 +1092,11 @@ def error(msg):
     
 # User-defined Displayable stuff.
 
-Displayable = renpy.display.core.Displayable
+# Displayable = renpy.display.core.Displayable
+
+class Displayable(renpy.display.core.Displayable, renpy.python.RevertableObject):
+    pass
+
 Render = renpy.display.render.Render
 render = renpy.display.render.render
 IgnoreEvent = renpy.display.core.IgnoreEvent
@@ -1087,11 +1113,18 @@ def scry():
 def munged_filename():
     return renpy.parser.munge_filename(get_filename_line()[0])
 
+loaded_modules = set()
+
 def load_module(name, **kwargs):
 
     if not renpy.game.init_phase:
         raise Exception("Module loading is only allowed in init code.")
 
+    if name in loaded_modules:
+        return
+
+    loaded_modules.add(name)
+    
     old_locked = renpy.config.locked
     renpy.config.locked = False
     
