@@ -84,7 +84,7 @@ def munge_filename(fn):
     rv = os.path.basename(fn)
     rv = os.path.splitext(rv)[0]
     rv = rv.replace(" ", "_")
-    return "__" + rv + "__"
+    return "_m1_" + rv + "__"
 
 def list_logical_lines(filename):
     """
@@ -215,7 +215,7 @@ def list_logical_lines(filename):
                 continue
 
             m = lllword.match(data, pos)
-
+            
             word = m.group(0)
             rest = m.group(1)
 
@@ -510,6 +510,8 @@ class Lexer(object):
             s = re.sub(r'\s+', ' ', s)
 
             s = s.replace("\\n", "\n")
+            s = s.replace("\\{", "{{")
+            s = s.replace("\\%", "%%")
             s = re.sub(r'\\u([0-9a-fA-F]{1,4})',
                        lambda m : unichr(int(m.group(1), 16)), s)
             s = re.sub(r'\\(.)', r'\1', s)
@@ -665,7 +667,7 @@ class Lexer(object):
 
             self.pos += 1
 
-        self.error("reached eol when expecting '%s'." % delim)
+        self.error("reached end of line when expecting '%s'." % delim)
 
     def python_expression(self):
         """
