@@ -79,7 +79,7 @@ style_properties = dict(
     aft_bar = none_is_null,
     aft_gutter = None,
     antialias = None,
-    background = renpy.easy.displayable,
+    background = renpy.easy.displayable_or_none,
     bar_invert = None,
     bar_resizing = None,
     bar_vertical = None,
@@ -88,7 +88,7 @@ style_properties = dict(
     bottom_margin = None,
     bottom_padding = None,
     box_layout = None,
-    child = renpy.easy.displayable,
+    child = renpy.easy.displayable_or_none,
     clipping = None,
     color = renpy.easy.color,
     drop_shadow = None,
@@ -100,7 +100,7 @@ style_properties = dict(
     font = None,
     fore_bar = none_is_null,
     fore_gutter = None,
-    foreground = renpy.easy.displayable,
+    foreground = renpy.easy.displayable_or_none,
     sound = None,
     italic = None,
     language = None,
@@ -345,26 +345,36 @@ def expand_properties(properties):
 
 # This builds the style. 
 def build_style(style):
-
+    
     if style.cache is not None:
         return
 
     updates = [ ]
-
+    
     if style.parent is not None:
-
+        
         name = style.parent
 
+        # The left base is the style that shares the most indexes with
+        # style.parent, while being one of the parents of style.parent.
         left_base = None
-        down_base = [ ]
 
         while True:
             first = name[0]
             rest = name[1:]
 
+            # The down bases inherit from the parents of the unindexd
+            # style, but don't have as many components.
+            
+            if rest:
+                down_base = [ style_map[first] ]
+            else:
+                down_base = [ ]
+            
             while first:
 
                 left_base = style_parts[first].get(rest, None)
+
                 if left_base:
                     break
 
@@ -376,7 +386,6 @@ def build_style(style):
                 break
 
             name = name[:-1]
-            down_base = [ ]
 
         for ss in down_base:
             if ss.updates:
@@ -387,6 +396,10 @@ def build_style(style):
                 updates.extend(ss.updates)
 
             for j in rest:
+
+                if ss.indexed is None:
+                    break
+                
                 ss = ss.indexed.get(j, None)
 
                 if ss is None:
@@ -401,10 +414,35 @@ def build_style(style):
             build_style(left_base)
 
         cache = left_base.cache
-                
+        
     else:
         cache = [ None ] * property_numbers
 
+    # Now, factor in the style that we're indexed off of.
+    if style.name is not None and len(style.name) > 1:
+
+        ss = style_map[style.name[0]]
+
+        if ss.cache is None:
+            build_style(ss)
+
+        updates.extend(ss.updates)
+        
+        for i in style.name[1:-1]:
+            if (ss.indexed is None) or (i not in ss.indexed):
+                break
+
+            ss = ss.indexed[i]
+            
+            if ss.cache is None:
+                build_style(ss)
+
+            updates.extend(ss.updates)
+            
+
+        
+            
+        
     style.updates = my_updates = [ ]
         
     for p in style.properties:
@@ -629,7 +667,7 @@ class Style(object):
 
         name = self.name + (index,)
         
-        s = Style(self.parent, name=name, heavy=not styles_built)
+        s = Style(self.parent + (index,), name=name, heavy=not styles_built)
 
         if not styles_built:
             self.indexed[index] = s
